"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "default", {
    enumerable: true,
    get: ()=>getModuleDependencies
});
const _fs = /*#__PURE__*/ _interopRequireDefault(require("fs"));
const _path = /*#__PURE__*/ _interopRequireDefault(require("path"));
const _resolve = /*#__PURE__*/ _interopRequireDefault(require("resolve"));
const _detectiveTypescript = /*#__PURE__*/ _interopRequireDefault(require("detective-typescript"));
function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
function createModule(file) {
    let source = _fs.default.readFileSync(file, "utf-8");
    return {
        file,
        requires: (0, _detectiveTypescript.default)(source, {
            mixedImports: true
        })
    };
}
function* _getModuleDependencies(entryFile) {
    yield entryFile;
    let mod = createModule(entryFile);
    let ext = _path.default.extname(entryFile);
    let isTypeScript = ext === ".ts" || ext === ".cts" || ext === ".mts";
    let extensions = [
        ...isTypeScript ? [
            ".ts",
            ".cts",
            ".mts"
        ] : [],
        ".js",
        ".cjs",
        ".mjs"
    ];
    // Iterate over the modules, even when new
    // ones are being added
    for (let dep of mod.requires){
        // Only track local modules, not node_modules
        if (!dep.startsWith("./") && !dep.startsWith("../")) {
            continue;
        }
        try {
            let basedir = _path.default.dirname(mod.file);
            let depPath = _resolve.default.sync(dep, {
                basedir,
                extensions
            });
            yield* _getModuleDependencies(depPath);
        } catch (_err) {
        // eslint-disable-next-line no-empty
        }
    }
}
function getModuleDependencies(entryFile) {
    return new Set(_getModuleDependencies(entryFile));
}
